import com.google.zxing.BarcodeFormat
import com.google.zxing.EncodeHintType
import com.google.zxing.MultiFormatWriter

import com.google.zxing.client.j2se.MatrixToImageWriter
import com.google.zxing.client.j2se.MatrixToImageConfig

import org.apache.commons.codec.binary.Base64
import org.apache.commons.io.IOUtils
import org.apache.log4j.Logger

barcode = new BarcodeTool()

/**
 * Velocity tool to generate various types of barcode and QR code (using the ZXing library).
 *
 * @see http://en.wikipedia.org/wiki/Barcode
 * @see http://github.com/zxing/zxing
 */
class BarcodeTool {
	def log = Logger.getLogger(this.getClass())

	final hints = [(EncodeHintType.CHARACTER_SET): "UTF-8", (EncodeHintType.MARGIN): 0]
	final matrixToImageConfig = new MatrixToImageConfig()

	/**
	 * Returns the barcode in Base64-encoded PNG format, using the default resolution.
	 */
	def generate(def content, def barcodeFormat) {
		generate(content, barcodeFormat, 100, 100)
	}

	/**
	 * Returns the barcode in Base64-encoded PNG format.
	 *
	 * @param content the data to encode in the barcode (most typically a URL).
	 * @param barcodeFormat barcode format to generate (see http://zxing.github.io/zxing/apidocs/com/google/zxing/BarcodeFormat.html).
	 * @param width image width in pixels.
	 * @param height image height in pixels.
	 */
	def generate(def content, def barcodeFormat, def width, def height) {
		def out
		try {
			def bitMatrix = new MultiFormatWriter().encode(content, barcodeFormat as BarcodeFormat, width, height, hints)
			out = new ByteArrayOutputStream()
			MatrixToImageWriter.writeToStream(bitMatrix, "PNG", out, matrixToImageConfig)
			return Base64.encodeBase64String(out.toByteArray())
		} catch (Throwable e) {
			log.error("Failed to generate barcode", e)
			return ""
		} finally {
			IOUtils.closeQuietly(out)
		}
	}
}
